@@ -189,6 +189,7 @@ class _StepdownThread(threading.Thread):  # pylint: disable=too-many-instance-at
             self._step_down(rs_fixture)
         self._is_idle_evt.set()
 
+    # pylint: disable=R0912,R0915
     def _step_down(self, rs_fixture):
         try:
             primary = rs_fixture.get_primary(timeout_secs=self._stepdown_interval_secs)
@@ -208,8 +209,11 @@ class _StepdownThread(threading.Thread):  # pylint: disable=too-many-instance-at
         if self._kill:
             self.logger.info("Killing the primary on port %d of replica set '%s'.", primary.port,
                              rs_fixture.replset_name)
+
+            # We send the mongod process the signal to exit but don't immediately wait for it to
+            # exit because clean shutdown may take a while and we want to restore write availability
+            # as quickly as possible.
             primary.mongod.stop(kill=True)
-            primary.mongod.wait()
         else:
             self.logger.info("Stepping down the primary on port %d of replica set '%s'.",
                              primary.port, rs_fixture.replset_name)
@@ -223,17 +227,25 @@ class _StepdownThread(threading.Thread):  # pylint: disable=too-many-instance-at
             except pymongo.errors.AutoReconnect:
                 # AutoReconnect exceptions are expected as connections are closed during stepdown.
                 pass
+            except pymongo.errors.ExecutionTimeout as err:
+                # ExecutionTimeout exceptions are expected when the election attempt fails due to
+                # not being able to acquire the global X lock within self._stepdown_duration_secs
+                # seconds. We'll try again after self._stepdown_interval_secs seconds.
+                self.logger.info(
+                    "Failed to step down the primary on port %d of replica set '%s': %s",
+                    primary.port, rs_fixture.replset_name, err)
+                return
             except pymongo.errors.PyMongoError:
                 self.logger.exception(
                     "Error while stepping down the primary on port %d of replica set '%s'.",
                     primary.port, rs_fixture.replset_name)
                 raise
 
-        # We pick arbitrary secondary to run for election immediately in order to avoid a long
+        # We pick an arbitrary secondary to run for election immediately in order to avoid a long
         # period where the replica set doesn't have write availability. If none of the secondaries
-        # are eligible, or their election attempt fails, then we'll simply not have write
-        # availability until the self._stepdown_duration_secs duration expires and 'primary' steps
-        # back up again.
+        # are eligible, or their election attempt fails, then we'll run the replSetStepUp command on
+        # 'primary' to ensure we have write availability sooner than the
+        # self._stepdown_duration_secs duration expires.
         while secondaries:
             chosen = random.choice(secondaries)
 
@@ -254,6 +266,11 @@ class _StepdownThread(threading.Thread):  # pylint: disable=too-many-instance-at
                 secondaries.remove(chosen)
 
         if self._kill:
+            self.logger.info("Waiting for the old primary on port %d of replica set '%s' to exit.",
+                             primary.port, rs_fixture.replset_name)
+
+            primary.mongod.wait()
+
             self.logger.info("Attempting to restart the old primary on port %d of replica set '%s.",
                              primary.port, rs_fixture.replset_name)
 
@@ -266,6 +283,27 @@ class _StepdownThread(threading.Thread):  # pylint: disable=too-many-instance-at
                 primary.await_ready()
             finally:
                 primary.preserve_dbpath = original_preserve_dbpath
+        else:
+            # We always run the {replSetFreeze: 0} command to ensure the former primary is electable
+            # in the next round of _step_down().
+            client = primary.mongo_client()
+            client.admin.command({"replSetFreeze": 0})
+
+        if not secondaries:
+            # If we failed to step up one of the secondaries, then we run the replSetStepUp to try
+            # and elect the former primary again. This way we don't need to wait
+            # self._stepdown_duration_secs seconds to restore write availability to the cluster.
+            try:
+                client = primary.mongo_client()
+                client.admin.command("replSetStepUp")
+            except pymongo.errors.OperationFailure as err:
+                # It is possible that by the time we've run the replSetStepUp command that
+                # self._stepdown_duration_secs seconds have already passed and the former primary is
+                # running for election on its own. We just ignore the error response from the former
+                # primary.
+                self.logger.info(
+                    "Failed to step up the old primary on port %d of replica set '%s': %s",
+                    primary.port, rs_fixture.replset_name, err)
 
         # Bump the counter for the chosen secondary to indicate that the replSetStepUp command
         # executed successfully.
