@@ -153,10 +153,6 @@ if (typeof _threadInject != "undefined") {
 
         // some tests can't run in parallel with most others
         var skipTests = makeKeys([
-            "repair.js",
-            "cursor8.js",
-            "recstore.js",
-            "extent.js",
             "indexb.js",
 
             // Tests that set a parameter that causes the server to ignore
@@ -164,15 +160,11 @@ if (typeof _threadInject != "undefined") {
             "index_bigkeys_nofail.js",
             "index_bigkeys_validation.js",
 
-            "mr_drop.js",
             "mr3.js",
-            "indexh.js",
             "evald.js",
-            "evalf.js",
             "run_program1.js",
             "notablescan.js",
             "bench_test1.js",
-            "padding.js",
 
             // These tests use the getLastError command, which is unsafe to use in this environment,
             // since a previous test's cursors could be garbage collected in the middle of the next
@@ -180,9 +172,14 @@ if (typeof _threadInject != "undefined") {
             "dropdb_race.js",
             "bulk_legacy_enforce_gle.js",
 
+            // These tests use getLog to examine the logs. Tests which do so shouldn't be run in
+            // this suite because any test being run at the same time could conceivably spam the
+            // logs so much that the line they are looking for has been rotated off the server's
+            // in-memory buffer of log messages, which only stores the 1024 most recent operations.
+            "getlog2.js",
+            "logprocessdetails.js",
             "queryoptimizera.js",
-            "loglong.js",  // log might overflow before
-            // this has a chance to see the message
+
             "connections_opened.js",  // counts connections, globally
             "opcounters_write_cmd.js",
             "set_param1.js",                  // changes global state
@@ -201,6 +198,35 @@ if (typeof _threadInject != "undefined") {
             // Views tests
             "views/invalid_system_views.js",  // Creates invalid view definitions in system.views.
             "views/views_all_commands.js",    // Drops test DB.
+
+            // Use eval command and potentially cause deadlock.
+            "apitest_db.js",
+            "constructors.js",
+            "error2.js",
+            "eval0.js",
+            "eval1.js",
+            "eval3.js",
+            "eval4.js",
+            "eval5.js",
+            "eval6.js",
+            "eval7.js",
+            "eval9.js",
+            "evala.js",
+            "evalb.js",
+            "evald.js",
+            "evale.js",
+            "evalg.js",
+            "eval_mr.js",
+            "eval_nolock.js",
+            "fsync.js",
+            "recursion.js",
+            "js3.js",
+            "js7.js",
+            "js9.js",
+            "remove8.js",
+            "rename4.js",
+            "shellkillop.js",
+            "storefunc.js",
         ]);
 
         // The following tests cannot run when shell readMode is legacy.
@@ -268,11 +294,6 @@ if (typeof _threadInject != "undefined") {
             parallelFilesDir + "/profile_repair_cursor.js",
             parallelFilesDir + "/profile_sampling.js",
             parallelFilesDir + "/profile_update.js",
-
-            // These tests use getLog to examine the slow query logs. Tests which examine the slow
-            // query logs can't be run concurrently with tests that affect the profile sampling
-            // rate, since that also impacts which operations get logged.
-            parallelFilesDir + "/getlog2.js",
         ];
         var serialTests = makeKeys(serialTestsArr);
 
@@ -313,6 +334,10 @@ if (typeof _threadInject != "undefined") {
         args.forEach(function(x) {
             print("         S" + suite + " Test : " + x + " ...");
             var time = Date.timeFunc(function() {
+                // Create a new connection to the db for each file. If tests share the same
+                // connection it can create difficult to debug issues.
+                db = new Mongo(db.getMongo().host).getDB(db.getName());
+                gc();
                 load(x);
             }, 1);
             print("         S" + suite + " Test : " + x + " " + time + "ms");
