@@ -1,7 +1,7 @@
 /**
  *    Copyright (C) 2009-2015 MongoDB Inc.
  *
- *    This program is free software: you can redistribute it and/or  modify
+ *    This program is free software: you can redistribute it and/or modify
  *    it under the terms of the GNU Affero General Public License, version 3,
  *    as published by the Free Software Foundation.
  *
@@ -414,10 +414,6 @@ mongo::BSONObj getBsonFromJsonFile(std::string fileName) {
     std::string filename(filePath.string());
     std::ifstream infile(filename.c_str());
     std::string data((std::istreambuf_iterator<char>(infile)), std::istreambuf_iterator<char>());
-    if (data.empty()) {
-        // v3.4 test infra is unable to see these extra files.
-        return mongo::BSONObj();
-    }
     mongo::BSONObj obj = mongo::fromjson(data);
     ASSERT_TRUE(obj.valid(mongo::BSONVersion::kLatest));
     ASSERT_TRUE(obj.hasField("tests"));
@@ -510,8 +506,8 @@ TEST(MongoURI, CloneURIForServer) {
 /**
  * These tests come from the Mongo Uri Specifications for the drivers found at:
  * https://github.com/mongodb/specifications/tree/master/source/connection-string/tests
- * They have been slighly altered as the Drivers specification is slighly different
- * from the server specification.
+ * They have been altered as the Drivers specification is somewhat different from the shell
+ * implementation.
  */
 TEST(MongoURI, specTests) {
     const std::string files[] = {
@@ -603,4 +599,278 @@ TEST(MongoURI, specTests) {
     }
 }
 
+TEST(MongoURI, srvRecordTest) {
+    using namespace mongo;
+    enum Expectation : bool { success = true, failure = false };
+    const struct {
+        int lineNumber;
+        std::string uri;
+        std::string user;
+        std::string password;
+        std::string database;
+        std::vector<HostAndPort> hosts;
+        std::map<std::string, std::string> options;
+        Expectation expectation;
+    } tests[] = {
+        // Test some non-SRV URIs to make sure that they do not perform expansions
+        {__LINE__,
+         "mongodb://test1.test.build.10gen.cc:12345/",
+         "",
+         "",
+         "",
+         {{"test1.test.build.10gen.cc", 12345}},
+         {},
+         success},
+
+        {__LINE__,
+         "mongodb://test6.test.build.10gen.cc:12345/",
+         "",
+         "",
+         "",
+         {{"test6.test.build.10gen.cc", 12345}},
+         {},
+         success},
+
+        // Test a sample URI against each provided testing DNS entry
+        {__LINE__,
+         "mongodb+srv://test1.test.build.10gen.cc/",
+         "",
+         "",
+         "",
+         {{"localhost.test.build.10gen.cc.", 27017}, {"localhost.test.build.10gen.cc.", 27018}},
+         {{"ssl", "true"}},
+         success},
+
+        // Test a sample URI against each provided testing DNS entry
+        {__LINE__,
+         "mongodb+srv://test1.test.build.10gen.cc/?ssl=false",
+         "",
+         "",
+         "",
+         {{"localhost.test.build.10gen.cc.", 27017}, {"localhost.test.build.10gen.cc.", 27018}},
+         {{"ssl", "false"}},
+         success},
+
+        // Test a sample URI against the need for deep DNS relation
+        {__LINE__,
+         "mongodb+srv://test18.test.build.10gen.cc/?replicaSet=repl0",
+         "",
+         "",
+         "",
+         {
+             {"localhost.sub.test.build.10gen.cc.", 27017},
+         },
+         {
+             {"ssl", "true"}, {"replicaSet", "repl0"},
+         },
+         success},
+
+        // Test a sample URI with FQDN against the need for deep DNS relation
+        {__LINE__,
+         "mongodb+srv://test18.test.build.10gen.cc./?replicaSet=repl0",
+         "",
+         "",
+         "",
+         {
+             {"localhost.sub.test.build.10gen.cc.", 27017},
+         },
+         {
+             {"ssl", "true"}, {"replicaSet", "repl0"},
+         },
+         success},
+
+        {__LINE__,
+         "mongodb+srv://user:password@test2.test.build.10gen.cc/"
+         "database?someOption=someValue&someOtherOption=someOtherValue",
+         "user",
+         "password",
+         "database",
+         {{"localhost.test.build.10gen.cc.", 27018}, {"localhost.test.build.10gen.cc.", 27019}},
+         {{"someOption", "someValue"}, {"someOtherOption", "someOtherValue"}, {"ssl", "true"}},
+         success},
+
+
+        {__LINE__,
+         "mongodb+srv://user:password@test3.test.build.10gen.cc/"
+         "database?someOption=someValue&someOtherOption=someOtherValue",
+         "user",
+         "password",
+         "database",
+         {{"localhost.test.build.10gen.cc.", 27017}},
+         {{"someOption", "someValue"}, {"someOtherOption", "someOtherValue"}, {"ssl", "true"}},
+         success},
+
+
+        {__LINE__,
+         "mongodb+srv://user:password@test5.test.build.10gen.cc/"
+         "database?someOption=someValue&someOtherOption=someOtherValue",
+         "user",
+         "password",
+         "database",
+         {{"localhost.test.build.10gen.cc.", 27017}},
+         {{"someOption", "someValue"},
+          {"someOtherOption", "someOtherValue"},
+          {"replicaSet", "repl0"},
+          {"authSource", "thisDB"},
+          {"ssl", "true"}},
+         success},
+
+        {__LINE__,
+         "mongodb+srv://user:password@test5.test.build.10gen.cc/"
+         "database?someOption=someValue&authSource=anotherDB&someOtherOption=someOtherValue",
+         "user",
+         "password",
+         "database",
+         {{"localhost.test.build.10gen.cc.", 27017}},
+         {{"someOption", "someValue"},
+          {"someOtherOption", "someOtherValue"},
+          {"replicaSet", "repl0"},
+          {"replicaSet", "repl0"},
+          {"authSource", "anotherDB"},
+          {"ssl", "true"}},
+         success},
+
+        {__LINE__, "mongodb+srv://test6.test.build.10gen.cc/", "", "", "", {}, {}, failure},
+
+        {__LINE__,
+         "mongodb+srv://test6.test.build.10gen.cc/database",
+         "",
+         "",
+         "database",
+         {},
+         {},
+         failure},
+
+        {__LINE__,
+         "mongodb+srv://test6.test.build.10gen.cc/?authSource=anotherDB",
+         "",
+         "",
+         "",
+         {},
+         {},
+         failure},
+
+        {__LINE__,
+         "mongodb+srv://test6.test.build.10gen.cc/?irrelevantOption=irrelevantValue",
+         "",
+         "",
+         "",
+         {},
+         {},
+         failure},
+
+
+        {__LINE__,
+         "mongodb+srv://test6.test.build.10gen.cc/"
+         "?irrelevantOption=irrelevantValue&authSource=anotherDB",
+         "",
+         "",
+         "",
+         {},
+         {},
+         failure},
+
+        {__LINE__,
+         "mongodb+srv://test7.test.build.10gen.cc./?irrelevantOption=irrelevantValue",
+         "",
+         "",
+         "",
+         {},
+         {},
+         failure},
+
+        {__LINE__, "mongodb+srv://test7.test.build.10gen.cc./", "", "", "", {}, {}, failure},
+
+        {__LINE__, "mongodb+srv://test8.test.build.10gen.cc./", "", "", "", {}, {}, failure},
+
+        {__LINE__,
+         "mongodb+srv://test10.test.build.10gen.cc./?irrelevantOption=irrelevantValue",
+         "",
+         "",
+         "",
+         {},
+         {},
+         failure},
+
+        {__LINE__,
+         "mongodb+srv://test11.test.build.10gen.cc./?irrelevantOption=irrelevantValue",
+         "",
+         "",
+         "",
+         {},
+         {},
+         failure},
+
+        {__LINE__, "mongodb+srv://test12.test.build.10gen.cc./", "", "", "", {}, {}, failure},
+        {__LINE__, "mongodb+srv://test13.test.build.10gen.cc./", "", "", "", {}, {}, failure},
+        {__LINE__, "mongodb+srv://test14.test.build.10gen.cc./", "", "", "", {}, {}, failure},
+        {__LINE__, "mongodb+srv://test15.test.build.10gen.cc./", "", "", "", {}, {}, failure},
+        {__LINE__, "mongodb+srv://test16.test.build.10gen.cc./", "", "", "", {}, {}, failure},
+        {__LINE__, "mongodb+srv://test17.test.build.10gen.cc./", "", "", "", {}, {}, failure},
+        {__LINE__, "mongodb+srv://test19.test.build.10gen.cc./", "", "", "", {}, {}, failure},
+
+        {__LINE__, "mongodb+srv://test12.test.build.10gen.cc/", "", "", "", {}, {}, failure},
+        {__LINE__, "mongodb+srv://test13.test.build.10gen.cc/", "", "", "", {}, {}, failure},
+        {__LINE__, "mongodb+srv://test14.test.build.10gen.cc/", "", "", "", {}, {}, failure},
+        {__LINE__, "mongodb+srv://test15.test.build.10gen.cc/", "", "", "", {}, {}, failure},
+        {__LINE__, "mongodb+srv://test16.test.build.10gen.cc/", "", "", "", {}, {}, failure},
+        {__LINE__, "mongodb+srv://test17.test.build.10gen.cc/", "", "", "", {}, {}, failure},
+        {__LINE__, "mongodb+srv://test19.test.build.10gen.cc/", "", "", "", {}, {}, failure},
+    };
+
+    for (const auto& test : tests) {
+        auto rs = MongoURI::parse(test.uri);
+        if (test.expectation == failure) {
+            ASSERT_FALSE(rs.getStatus().isOK()) << "Failing URI: " << test.uri
+                                                << " data on line: " << test.lineNumber;
+            continue;
+        }
+        ASSERT_OK(rs.getStatus()) << "Failed on URI: " << test.uri
+                                  << " data on line: " << test.lineNumber;
+        auto rv = rs.getValue();
+        ASSERT_EQ(rv.getUser(), test.user) << "Failed on URI: " << test.uri
+                                           << " data on line: " << test.lineNumber;
+        ASSERT_EQ(rv.getPassword(), test.password) << "Failed on URI: " << test.uri
+                                                   << " data on line : " << test.lineNumber;
+        ASSERT_EQ(rv.getDatabase(), test.database) << "Failed on URI: " << test.uri
+                                                   << " data on line : " << test.lineNumber;
+        std::vector<std::pair<std::string, std::string>> options(begin(rv.getOptions()),
+                                                                 end(rv.getOptions()));
+        std::sort(begin(options), end(options));
+        std::vector<std::pair<std::string, std::string>> expectedOptions(begin(test.options),
+                                                                         end(test.options));
+        std::sort(begin(expectedOptions), end(expectedOptions));
+
+        for (std::size_t i = 0; i < std::min(options.size(), expectedOptions.size()); ++i) {
+            if (options[i] != expectedOptions[i]) {
+                mongo::unittest::log() << "Option: \"" << options[i].first << "="
+                                       << options[i].second << "\" doesn't equal: \""
+                                       << expectedOptions[i].first << "="
+                                       << expectedOptions[i].second << "\""
+                                       << " data on line: " << test.lineNumber << std::endl;
+                std::cerr << "Failing URI: \"" << test.uri << "\""
+                          << " data on line: " << test.lineNumber << std::endl;
+                ASSERT(false);
+            }
+        }
+        ASSERT_EQ(options.size(), expectedOptions.size()) << "Failing URI: "
+                                                          << " data on line: " << test.lineNumber
+                                                          << test.uri;
+
+        std::vector<HostAndPort> hosts(begin(rv.getServers()), end(rv.getServers()));
+        std::sort(begin(hosts), end(hosts));
+        auto expectedHosts = test.hosts;
+        std::sort(begin(expectedHosts), end(expectedHosts));
+
+        for (std::size_t i = 0; i < std::min(hosts.size(), expectedHosts.size()); ++i) {
+            ASSERT_EQ(hosts[i], expectedHosts[i]) << "Failed on URI: " << test.uri
+                                                  << " at host number" << i
+                                                  << " data on line: " << test.lineNumber;
+        }
+        ASSERT_TRUE(hosts.size() == expectedHosts.size())
+            << "Failed on URI: " << test.uri << " Found " << hosts.size() << " hosts, expected "
+            << expectedHosts.size() << " data on line: " << test.lineNumber;
+    }
+}
+
 }  // namespace
