@@ -378,7 +378,7 @@ public:
     // Produce member data for the serverStatus command and diagnostic logging.
     void fillMemberData(BSONObjBuilder* result);
 
-    enum class PrepareFreezeResponseResult { kNoAction, kElectSelf };
+    enum class PrepareFreezeResponseResult { kNoAction, kSingleNodeSelfElect };
 
     /**
      * Produce a reply to a freeze request. Returns a PostMemberStateUpdateAction on success that
@@ -649,6 +649,13 @@ public:
      */
     void finishUnconditionalStepDown();
 
+    /**
+     * Returns the index of the most suitable candidate for an election handoff. The node must be
+     * caught up and electable. Ties are resolved first by highest priority, then by lowest member
+     * id.
+     */
+    int chooseElectionHandoffCandidate();
+
     /**
      * Considers whether or not this node should stand for election, and returns true
      * if the node has transitioned to candidate role as a result of the call.
@@ -705,7 +712,8 @@ public:
         kElectionTimeout,
         kPriorityTakeover,
         kStepUpRequest,
-        kCatchupTakeover
+        kCatchupTakeover,
+        kSingleNodePromptElection
     };
 
     /**
@@ -878,6 +886,11 @@ private:
      */
     bool _canCompleteStepDownAttempt(Date_t now, Date_t waitUntil, bool force);
 
+    /**
+     * Returns true if a node is both caught up to our last applied opTime and electable.
+     */
+    bool _isCaughtUpAndElectable(int memberIndex, OpTime lastApplied);
+
     void _stepDownSelfAndReplaceWith(int newPrimary);
 
     /**
